;**************************************************************************
;*
;* FILE  diagnostics.asm
;* Copyright (c) 2006, 2007, 2008 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: diagnostics.asm,v 1.19 2008-02-17 21:56:46 tlr Exp $
;*
;* DESCRIPTION
;*   prototype code to check flash type and dtv2/dtv3
;*
;******

	seg.u	zp_diag
	org	$fa
;**************************************************************************
;*
;* SECTION  zp_diag
;*
;* DESCRIPTION
;*   Zeropage area allocation
;*
;******
zp_diag_start:
diagptr_zp:
	ds.w	1
diagptr2_zp:
	ds.w	1
diagzero_zp:
	ds.b	1
diagtmp_zp:
	ds.b	1
zp_diag_end:
	echo	"[diagnostics] zp usage:",zp_diag_start,"-",zp_diag_end

	seg.u	stack_diag
	org	$0108
;**************************************************************************
;*
;* SECTION  stack_diag
;*
;* DESCRIPTION
;*   Stack area allocation
;*
;******
stack_diag_start:
is_dtv3:
	ds.b	1
is_pal:
	ds.b	1
flash_type:
	ds.b	1
scan_tab:
	ds.b	64
scan_dc00:	; joy2
	ds.b	1
scan_dc01:	; joy1
	ds.b	1
stack_diag_end:
	echo	"[diagnostics] stack usage:",stack_diag_start,"-",stack_diag_end
	

	seg	code
;**************************************************************************
;*
;* SysAddress... When run we will enter here!
;*
;******
diagnostics:
	jsr	$ff5b
	sei

	lda	#1
	sta	$d03f

; setup colors
PALETTE	equ	$fa56
	lda	PALETTE+6
	sta	$d020
;	lda	PALETTE+6
	sta	$d021
	lda	PALETTE+7
	sta	646
	ldx	#0
diag_lp1:
	sta	$d800,x
	sta	$d900,x
	sta	$da00,x
	sta	$dae8,x
	inx
	bne	diag_lp1

	lda	#0
	sta	is_dtv3
	jsr	check_blit
	bcs	diag_skp1
	lda	#1
	sta	is_dtv3
diag_skp1:

	lda	#0
	sta	is_pal
	jsr	check_pal
	bcc	diag_skp2
	lda	#1
	sta	is_pal
diag_skp2:
	
	jsr	identify_flash
	sta	flash_type

	jsr	display_test

; print heading
	lda	#<heading_msg
	ldy	#>heading_msg
	jsr	print_xy

; print DTV type
	lda	#<type_msg
	ldy	#>type_msg
	jsr	print_xy
	lda	is_dtv3
	clc
	adc	#$32
	jsr	$ffd2
	
; print video mode
	lda	#<vmode_pal_msg
	ldy	#>vmode_pal_msg
	ldx	is_pal
	bne	diag_skp3
	lda	#<vmode_ntsc_msg
	ldy	#>vmode_ntsc_msg
diag_skp3:	
	jsr	print_xy

; print flash type
	lda	#<flash_msg
	ldy	#>flash_msg
	jsr	print_xy
	lda	flash_type
	asl
	tax
	lda	flash_msg_tab,x
	ldy	flash_msg_tab+1,x
	jsr	print_xy

	jmp	keyboard_and_joystick_scan	

heading_msg:
	dc.b	1,3,24,"- DTVBOOT DIAGNOSTICS V1.2 / TLR -",0	
	
type_msg:
	dc.b	1,1,19,"TYPE : DTV",0	

vmode_pal_msg:
	dc.b	1,1,20,"MODE : PAL",0
vmode_ntsc_msg:
	dc.b	1,1,20,"MODE : NTSC",0

flash_msg:
	dc.b	1,1,21,"FLASH: ",0
flash_msg_tab:
	dc.w	not_found_msg
	dc.w	unknown_msg
	dc.w	at47_msg
	dc.w	at47t_msg
	dc.w	sst39vf1681_msg
	dc.w	sst39vf1682_msg
	
not_found_msg:
	dc.b	"NOT DETECTED!",0
unknown_msg:
	dc.b	"UNKNOWN TYPE",0
at47_msg:
	dc.b	"AT4XBV16X",0
at47t_msg:
	dc.b	"AT4XBV16XT",0
sst39vf1681_msg:
	dc.b	"SST39VF1681",0
sst39vf1682_msg:
	dc.b	"SST39VF1682",0

;**************************************************************************
;*
;* NAME  print_xy
;*			
;* DESCRIPTION
;*   print with added positioning.
;*   1,x,y sets cursor position
;*   
;******
print_xy:
	sta	diagptr_zp
	sty	diagptr_zp+1
prtx_lp1:
	jsr	prtx_get
	beq	prtx_ex1
	cmp	#1
	bne	prtx_skp1
	jsr	prtx_get
	pha
	jsr	prtx_get
	tax			
	pla
	tay
	clc
	jsr	$fff0
	bra	prtx_lp1
prtx_skp1:
	jsr	$ffd2
	bra	prtx_lp1
prtx_ex1:
	rts
prtx_get:
	ldy	#0
	lda	(diagptr_zp),y
	php
	inc	diagptr_zp
	bne	prtx_skp2
	inc	diagptr_zp+1
prtx_skp2:
	plp
	rts
	
;**************************************************************************
;*
;* NAME  check_blit
;*
;* DESCRIPTION
;*   Check if DTV2/3 blitting in transparent mode works 
;*   C=1 means broken, C=0 means working
;*
;*   
;******
check_blit:
	ldx	#$1f
	stx	diagtmp_zp
	lda	#0
	sta	diagzero_zp
cb_lp1:
	sta	$d320,x
	dex
	bpl	cb_lp1

	lda	#diagzero_zp
	sta	$d320		; Src A
	lda	#diagtmp_zp
	sta	$d330		; Dest D
	lda	#1
	sta	$d338		; blit length
	lda	#%00000101
	sta	$d33b		; transparent, Src B=0

; perform blit
	lda	#%00001111
	sta	$d33a
cb_lp2:
	lda	$d33f
	lsr
	bcs	cb_lp2
	lda	diagtmp_zp
	beq	cb_fl1
	rts
cb_fl1:
	sec
	rts

	if	0	; already exists in boot.asm
;**************************************************************************
;*
;* NAME  check_pal
;*
;* DESCRIPTION
;*   Check if pal.
;*   C=1 means pal
;*
;*   
;******
check_pal:
;Wait for the negative part of the screen
ww_lp1:
	lda	$d011
	bmi	ww_lp1
ww_lp2:
	lda	$d011
	bpl	ww_lp2
;check for pal
cp_lp1:
	lda	$d012
	cmp	#$20
	beq	cp_ex1		; C=1
	lda	$d011
	bmi	cp_lp1
	clc
cp_ex1:
	rts
	endif

;**************************************************************************
;*
;* NAME  identify_flash
;*
;* DESCRIPTION
;*   Try to identify the type of flash
;*
;******
IDENT_BUF_LEN	equ	32
CHARSET_PAGE	equ	$38	; $003800-$004000 (doesn't matter)
FLASH_WINDOW	equ	$8000
; Atmel $555*2 (A-1 is don't care)
; SST   $aaa
; i.e $aaa should work on both.
FLASH_OFFS_555	equ	$aaa
			
; Atmel $aaa*2 or $2aa*2 (A-1 is don't care)
; SST   $555
; i.e $555 should work on both.
FLASH_OFFS_AAA	equ	$555

identify_flash:
	sei

	ldx	#FLASHCHECK_LEN
idf_lp1:
	lda	flashcheck_st-1,x
	sta	flashcheck-1,x	
	dex
	bne	idf_lp1

; disable bad lines
	lda	#%00100000
	sta	$d03c
; point DMA to RAM.
	lda	#$40
	sta	$d302
	sta	$d305
; enable skip + burst
	sir	#$92
	ldy	#%00000011
; map flash to $8000-$c000
	sir	#$E8
	ldy	#0
	txa
	and	#%11001111
	tax
	sir	#$12
; wait until outside the screen
idf_lp2:
	lda	$d011
	bmi	idf_lp2
idf_lp3:
	lda	$d011
	bpl	idf_lp3
; move away the character fetch to RAM (not visible)
	lda	#%00010000 | ((CHARSET_PAGE / 8) * 2)
	sta	$d018

; perform flash check (on the stack)
	jsr	flashcheck

; restore normal mapping of $8000 to $c000
	sir	#$E8
	ldy	#$02
	txa
	ora	#%00010000
	tax
; disable skip + burst
	sir	#$92
	ldy	#%00000000
	sir	#$12
; reenable badlines
	lda	#%00000000
	sta	$d03c	
; restore charset
	lda	#%00010100
	sta	$d018
	
; check for differences
	ldy	#IDENT_BUF_LEN-1
idf_lp4:
	lda	ident_buf,y
	cmp	ident_buf_pre,y
	bne	idf_skp7	; diff, yes skip out.
	dey
	bpl	idf_lp4
; no bytes changed = no flash found
	ldx	#TYPE_NOT_FOUND
	bra	idf_ex1

; The data is from product id mode, check type
idf_skp7:
	ldx	#TYPE_UNKNOWN

	lda	id1
	cmp	#$1f		; Atmel
	beq	idf_skp3
	cmp	#$bf		; SST
	bne	idf_ex1
	
; SST Flashes
	lda	id2sst
	cmp	#$c8		; SST39VF1681
	bne	idf_skp4
	ldx	#TYPE_SST39VF1681
idf_skp4:
	cmp	#$c9		; SST39VF1682
	bne	idf_skp5
	ldx	#TYPE_SST39VF1682
idf_skp5:
	bra	idf_ex1

; Atmel Flashes
idf_skp3:
	lda	id2at
	cmp	#$c0
	bne	idf_skp1
	ldx	#TYPE_AT4XBV16X
idf_skp1:
	cmp	#$c2		; top bootblock
	bne	idf_skp2
	ldx	#TYPE_AT4XBV16XT
idf_skp2:
idf_ex1:
	txa
	rts
		
TYPE_NOT_FOUND		equ	0
TYPE_UNKNOWN		equ	1
TYPE_AT4XBV16X		equ	2
TYPE_AT4XBV16XT		equ	3
TYPE_SST39VF1681	equ	4
TYPE_SST39VF1682	equ	5


;**************************************************************************
;*
;* NAME  flashcheck
;*
;* DESCRIPTION
;*   flash checking part to run on stack.
;*
;******
flashcheck_st:
	rorg	$0110
flashcheck:
	ldx	#0
; read before product id mode so we can see later if product id was entered.
	jsr	fc_copy

; enter product id mode
	lda	#$aa
	sta	FLASH_WINDOW+FLASH_OFFS_555
	lda	#$55
	sta	FLASH_WINDOW+FLASH_OFFS_AAA
	lda	#$90
	sta	FLASH_WINDOW+FLASH_OFFS_555

; read during product id mode
	jsr	fc_copy

; exit product id mode
	lda	#$f0
	sta	FLASH_WINDOW
	rts

fc_copy:
	ldy	#0
fc_lp2:
	lda	FLASH_WINDOW,y
	sta	ident_buf_pre,x
	inx
	iny
	cpy	#IDENT_BUF_LEN
	bne	fc_lp2
	rts

ident_buf_pre:
	ds.b	IDENT_BUF_LEN
ident_buf:
	ds.b	IDENT_BUF_LEN
id1	equ	ident_buf+0*2
id2at	equ	ident_buf+1*2
id2sst	equ	ident_buf+1
id3at	equ	ident_buf+3*2
flashcheck_end:
	rend
FLASHCHECK_LEN	equ	.-flashcheck_st
	echo	"[diagnostics] flash check:",flashcheck,"-",flashcheck_end	

;**************************************************************************
;*
;* NAME  display_test
;*
;* DESCRIPTION
;*   Show colour test pattern
;*
;******
DT_X	equ	0
DT_Y	equ	0
DT_BASE	equ	$0400+40*DT_Y+DT_X
display_test:
	lda	#%00100100
	sta	$d03c
	ldy	#16-1
dt_lp1:
	tya
	sta	$d200,y	 
	dey
	bpl	dt_lp1

	lda	#0
	sta	diagtmp_zp
	lda	#<DT_BASE
	sta	diagptr_zp
	sta	diagptr2_zp
	lda	#>DT_BASE
	sta	diagptr_zp+1
	lda	#(>DT_BASE)+($d8-$04)
	sta	diagptr2_zp+1
	ldx	#16-1
dt_lp2:
	ldy	#0
dt_lp3:	
	lda	#$a0
	sta	(diagptr_zp),y
	lda	diagtmp_zp
	sta	(diagptr2_zp),y		
	inc	diagtmp_zp
	iny
	cpy	#16
	bne	dt_lp3
	lda	diagptr_zp
	clc
	adc	#40
	sta	diagptr_zp
	sta	diagptr2_zp
	lda	diagptr_zp+1
	adc	#0
	sta	diagptr_zp+1
	adc	#$d8-$04
	sta	diagptr2_zp+1
	dex
	bpl	dt_lp2

	rts	


;**************************************************************************
;*
;* NAME  keyboard_and_joystick_scan
;*
;* DESCRIPTION
;*   scan keyboard and joysticks.
;*   Never returns.
;*
;******
keyboard_and_joystick_scan:
	lda	#<key_msg
	ldy	#>key_msg
	jsr	print_xy

	sei
kjs_lp1:
	jsr	perform_scan
	jsr	show_keymatrix

	lda	scan_dc01
	ldy	#0
	jsr	show_joystick

	lda	scan_dc00
	ldy	#7
	jsr	show_joystick
	
	bra	kjs_lp1
	

LEFT_ARROW	equ	31
UP_ARROW	equ	30
ANGLE		equ	125
BALL		equ	81
	
KS_X	equ	23
KS_Y	equ	1
KS_BASE	equ	$0400+[[KS_Y+3]*40]+[KS_X+7]

JS_X	equ	KS_X+5
JS_Y	equ	KS_Y+15
JS_BASE	equ	$0400+JS_Y*40+JS_X

key_msg:
	dc.b	1,KS_X+9,KS_Y+0,          "$DC01"
	dc.b	1,KS_X+9,KS_Y+1,          "READ"
	dc.b	1,KS_X+7,KS_Y+2,        "76543210-"
	dc.b	1,KS_X+6,KS_Y+3,       "7"
	dc.b	1,KS_X+6,KS_Y+4,       "6"
	dc.b	1,KS_X+6,KS_Y+5,       "5"
	dc.b	1,KS_X+0,KS_Y+6, "$DC00 4"
	dc.b	1,KS_X+0,KS_Y+7, "WRITE 3"
	dc.b	1,KS_X+6,KS_Y+8,       "2"
	dc.b	1,KS_X+6,KS_Y+9,       "1"
	dc.b	1,KS_X+6,KS_Y+10,      "0"
	dc.b	1,KS_X+6,KS_Y+11,      "-"
	dc.b	1,JS_X+2,JS_Y+2, "1"
	dc.b	1,JS_X+9,JS_Y+2, "2"
	dc.b	0

;**************************************************************************
;*
;* NAME  perform_scan
;*
;* DESCRIPTION
;*   Scan keyboard matrix (in both directions) + read $dc00 and $dc01
;*   with the other port as all $ff.
;*
;******
perform_scan:
	lda	#%00000000
	sta	$dc02
	lda	#%11111111
	sta	$dc03

	ldx	#7
	lda	#%11111110
pscn_lp1:
	pha
	sta	$dc01
	bit	$dc00
	lda	$dc00
	sta	diagtmp_zp
	ldy	#7
pscn_lp2:
	lda	#0
	lsr	diagtmp_zp
	bcs	pscn_skp1
	lda	#2
pscn_skp1:
	sta	scan_tab,x
	txa
	clc
	adc	#8
	tax
	dey
	bpl	pscn_lp2
	txa
	sec
	sbc	#64+1
	tax
	pla
	sec
	rol
	bcs	pscn_lp1
	sta	$dc01	; =$ff
	bit	$dc00
	lda	$dc00
	sta	scan_dc00
	
	lda	#%11111111
	sta	$dc02
	lda	#%00000000
	sta	$dc03

	ldx	#0
	lda	#%11111110
pscn_lp3:
	pha
	sta	$dc00
	bit	$dc01
	lda	$dc01
	ldy	#7
pscn_lp4:
	asl
	bcs	pscn_skp2
	inc	scan_tab,x
pscn_skp2:
	inx
	dey
	bpl	pscn_lp4

	pla
	sec
	rol
	bcs	pscn_lp3
	sta	$dc00	; =$ff
	bit	$dc01
	lda	$dc01
	sta	scan_dc01

	rts

;**************************************************************************
;*
;* NAME  show_keymatrix
;*
;* DESCRIPTION
;*   Show keymatrix bits on screen.
;*
;******
show_keymatrix:	
	lda	#<[KS_BASE+40*7]
	sta	diagptr_zp
	lda	#>[KS_BASE+40*7]
	sta	diagptr_zp+1
	ldx	#0
skm_lp1:
	ldy	#0
skm_lp2:
	txa
	pha
	lda	scan_tab,x
	tax
	lda	scan_ptab,x
	sta	(diagptr_zp),y
	pla
	tax
	inx
	iny
	cpy	#8
	bne	skm_lp2
	lda	diagptr_zp
	sec
	sbc	#40
	sta	diagptr_zp
	bcs	skm_skp1
	dec	diagptr_zp+1
skm_skp1:
	cpx	#64
	bne	skm_lp1

; show clean scan ($dc01)
	ldx	#7
	lda	scan_dc01
skm_lp3:
	lsr
	pha
	bcc	skm_skp2
	lda	#"."
	dc.b	$2c
skm_skp2:
	lda	#UP_ARROW
	sta	KS_BASE+[40*8],X	
	pla
	dex
	bpl	skm_lp3

; show clean scan ($dc00)
	lda	#<[KS_BASE+8]
	sta	diagptr_zp
	lda	#>[KS_BASE+8]
	sta	diagptr_zp+1
	ldy	#0
	ldx	#7
	lda	scan_dc00
skm_lp4:
	asl
	pha
	bcc	skm_skp3
	lda	#"."
	dc.b	$2c
skm_skp3:
	lda	#LEFT_ARROW
	sta	(diagptr_zp),y
	lda	diagptr_zp
	clc
	adc	#40
	sta	diagptr_zp
	bcc	skm_skp4
	inc	diagptr_zp+1
skm_skp4:
	pla
	dex
	bpl	skm_lp4
	
	rts
	
scan_ptab:
	dc.b	".",UP_ARROW,LEFT_ARROW,ANGLE


;**************************************************************************
;*
;* NAME  show_joystick
;*
;* DESCRIPTION
;*   Show joystick bits on screen.
;*   Acc = bits, Y=screen offset
;*
;******
show_joystick:
	sta	diagtmp_zp
	tya
	clc
	adc	#<JS_BASE
	sta	diagptr_zp
	lda	#>JS_BASE
	adc	#0
	sta	diagptr_zp+1

	ldx	#4
sj_lp1:
	lsr	diagtmp_zp
	bcs	sj_skp1
	lda	#BALL
	dc.b	$2c
sj_skp1:
	lda	#"."
	ldy	scan_jtab,x
	sta	(diagptr_zp),y
	dex
	bpl	sj_lp1

	rts
		
scan_jtab:
	dc.b	0,84,80,162,2
	
; eof
